-- https://adventofcode.com/2024/day/2

{-# LANGUAGE OverloadedStrings #-}

import           Data.List    (all, drop, elem, filter, length, map, or, sort,
                               sortOn, tail, take, zipWith)
import           Data.Ord     (Down (Down), Ord ((<=), (>=)))
import           Data.Text    (Text, lines, pack, splitOn, unpack)
import           Data.Text.IO (readFile)
import           GHC.IO       (unsafePerformIO)
import           Prelude      hiding (lines, readFile)

entries :: [[Int]]
entries = parse . unsafePerformIO $ lines <$> readFile "src/advent_of_code/2024/2.input"

entries' :: [[Int]]
entries' =  parse . unsafePerformIO $ lines <$> readFile "src/advent_of_code/2024/2f.input"

parse :: [Text] -> [[Int]]
parse = map (map (read . unpack) . splitOn " ")

main :: IO ()
main = do
  entries <- parse . lines <$> readFile "src/advent_of_code/2024/2.input"
  print "Advent of Code 2024 - Day 2"

  print $ "Part 1: " <> show (solveP1 entries)
  print $ "Part 2: " <> show (solveP2 entries)

solveP1 :: [[Int]] -> Int
solveP1 = length . filter (== True) . map safeReportP1

solveP2 :: [[Int]] -> Int
solveP2 = length . filter (== True) . map safeReportP2

safeReportP1 :: [Int] -> Bool
safeReportP1 report = isSorted report && adjacentLevelCheck report

isSorted :: [Int] -> Bool
isSorted report = report `elem` [sort report, sortOn Down report]

adjacentLevelCheck :: [Int] -> Bool
adjacentLevelCheck report =
  all (((== True) . (\n-> n>=1 && n<=3)) . abs)
      (zipWith subtract report (tail report))

safeReportP2 :: [Int] -> Bool
safeReportP2 report = any safeReportP1 (removeOneBadReport report)

removeOneBadReport :: [Int] -> [[Int]]
removeOneBadReport lst = map (\i -> take i lst <> drop (succ i) lst) [0..length lst-1]
